/*-
 * Copyright (c) 2011 Allan Saddi <allan@saddi.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#ifndef _APS_PREFORKSERVER_H
#define _APS_PREFORKSERVER_H

#include <sys/types.h>
#include <sys/select.h>
#include <sys/time.h>
#include <unistd.h>

/* The client handler run by children. parent descriptor allows for checking
   the state of the parent (should not be written to, only check for EOF).
   The client descriptor is the client's socket and should be closed when
   done. */
typedef void (*prefork_handler_t)(int parent, int client);

struct _ChildData;

struct _PreforkContext {
  prefork_handler_t handler;
  time_t maxIdle;

  int childCount;
  struct _ChildData **children; /* hash table */
};

typedef struct _PreforkContext PreforkContext;

/* Creates a new prefork server context, using the specified handler and
   max-idle value (for non-eternal children). Returns NULL on failure. */
PreforkContext *prefork_init(prefork_handler_t handler, time_t maxIdle);

/* Return total number of child processes */
int prefork_child_count(PreforkContext *ctxt);

/* Create a new child process. If eternal is non-zero, child will be
   permanent. Returns -1 if an error occurs. */
int prefork_new_child(PreforkContext *ctxt, int eternal);

/* Fill an fd_set for use with select(2). Returns max descriptor number + 1 */
int prefork_fill_fd_set(PreforkContext *ctxt, fd_set *fds);

/* Process children communication, using the passed in fd_set as returned
   from select(2). Also reaps any dead children. Returns the number of
   children that have just become available, if any. */
int prefork_process_children(PreforkContext *ctxt, fd_set *fds);

/* Handoff a client descriptor to the first available child. Returns
   -1 if an error occurs. errno will be EAGAIN if there were no more free
   children. */
int prefork_run(PreforkContext *ctxt, int fd);

/* Shutdown and free a prefork server context */
void prefork_shutdown(PreforkContext *ctxt);

#endif /* !_APS_PREFORKSERVER_H */
