/*-
 * Copyright (c) 2011 Allan Saddi <allan@saddi.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#ifndef APS_SCGI_H
#define APS_SCGI_H

#include <stdint.h>

#define SCGI_BUFFER_SIZE	8192

struct _SCGIContext;
struct _SCGIRequest;

typedef int (*scgi_handler_t)(struct _SCGIContext *ctxt);
typedef int (*scgi_body_handler_t)(struct _SCGIContext *ctxt, uint8_t *data,
				   size_t len, void *tofree);

typedef struct _SCGIContext {
  FILE *sock;
  scgi_handler_t handler;
  scgi_body_handler_t bodyHandler;
  uint8_t *bodyBuf;
  struct _SCGIRequest *activeReq;

  void *data; /* Application data, untouched by SCGI code */
} SCGIContext;

/* Common return codes */
#define SCGI_OK			0
#define SCGI_DONE		1 /* request done */
/* codes < 0 are errors */
#define SCGI_ERROR		-1
#define SCGI_MEMORY_ERROR	-2
#define SCGI_SOCKET_ERROR	-3
#define SCGI_PROTOCOL_ERROR	-4
#define SCGI_FILE_ERROR		-5

typedef struct _SCGIRequest {
  SCGIContext *context;
  char *rawHeaders;
  int numHeaders;
  const char **headers;

  /* Convenience */
  int contentLength;
  void *data; /* Application data, untouched by SCGI code */

  /* Private */
  int _bodyBytesAvail;
} SCGIRequest;

SCGIContext *scgiCreateContext(int sock, scgi_handler_t handler,
			       scgi_body_handler_t bodyHandler);
void scgiDestroyContext(SCGIContext *ctxt);

int scgiProcessInput(SCGIContext *ctxt);

int scgiSendHeaders(SCGIContext *ctxt,
		    uint16_t statusCode, const char *statusMsg,
		    int headerCount, const char **headers);
int scgiSendBodyChunk(SCGIContext *ctxt, const uint8_t *data, size_t len);
int scgiSendFile(SCGIContext *ctxt, int fd);
int scgiGetBodyChunk(SCGIContext *ctxt);

#endif /* APS_SCGI_H */
